/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/****  Project : eXtended C Library (XCLIB)
    *  Purpose : SYNCHRONOUS I/O MULTIPLEXING
    *  Language: ANSI C
    *  File    : demo_io-4.c
    *  Author  : Martin Vicente <martin@vicente.org>
    *
    ****/


/*****************************************************************************\

 [English]

 [French]

 Exemple naf d'un dbut d'embryon de toolkit X.

\*****************************************************************************/


#undef	__TRACE__
#define	__TRACE__

#ifdef X_DISPLAY_MISSING

#include <stdio.h>

int main()
{
    puts("X missing...");

    return 0;
}

#else

#include <stddef.h>

#include <X11/Xlib.h>

#include <xclib.h>

static void display_checker(IOCheckerId id, IOData *display);
static void display_handler(IOHandlerId id, int fd, IOEvent op, IOData *display);

int main()
{
    char    *display_string;
    Display *display;
    int      screen_num;
    Window   window;

    display_string = NULL;

    display = XOpenDisplay(display_string);

    if (display == NULL) error1("Can't open display \"%s\"", XDisplayName(display_string));

    io_add_checker(display_checker, display);
    io_add_handler(ConnectionNumber(display), ioREAD, display_handler, display);

    screen_num = DefaultScreen(display);

    window = XCreateSimpleWindow(
	display, RootWindow(display, screen_num),
	0, 0, 400, 300, 0,
	BlackPixel(display, screen_num), BlackPixel(display, screen_num));

    XSelectInput(display, window, StructureNotifyMask | ExposureMask);

    XMapWindow(display, window);

    io_main_loop();

    XCloseDisplay(display);

    return 0;
}

static void display_checker(IOCheckerId id, IOData *_display)
{
    Display *const display = (Display *)_display;

    while (XPending(display))
    {
	XEvent event;

	XNextEvent(display, &event);

	puts(__FUNC__);
    }
}

static void display_handler(IOHandlerId id, int fd, IOEvent op, IOData *_display)
{
    Display *const display = (Display *)_display;

    do
    {
	XEvent event;

	XNextEvent(display, &event);

	puts(__FUNC__);
    }
    while (XPending(display));
}

#endif


/* End of file */
