/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/****  Project : eXtended C Library (XCLIB)
    *  Purpose : SOME USEFUL MACROS
    *  Language: ANSI C
    *  File    : demo_macros.c
    *  Author  : Martin Vicente <martin@vicente.org>
    *
    ****/


#define _XCLIB_SOURCE

#include <stdio.h>

#include <xc/colors.h>
#include <xc/macros.h>

#define PI	_XC_PI

#define B1	SGR_BOLD
#define B0	SGR_BOLD_OFF

#define PRINT(M, X, F1, F2, N)\
 printf("%*s= %" #F2 "\r" B1 #M B0 "(%" #F1 ")\n", N, "", M (X), X)

#define COMM(en, fr)	__xcint(comm)(en, fr, NULL)
#define WAIT()		__xcint(wait)()

extern void __xcint(comm)(const char *en, ...);	/* DON'T USE IT!!! */
extern void __xcint(wait)(void);		/* DON'T USE IT!!! */

int main()
{
    double array[5];
    float  matrix[2][3];
    int    a, b;
    double x, y, z;

    COMM("SOME USEFUL MACROS",
	 "QUELQUES MACROS UTILES");

    fputs("For ever...", stdout);

    fflush(stdout);

    for ever
    {
	if (1)
	{
	    puts(" Break!");

	    break;
	}
    }

    printf("Cardinal of array   : %u\n", cardinal(array));
    printf("Dimensions of matrix: %u x %u\n", cardinal(matrix), cardinal(matrix[0]));

    WAIT();

    COMM("SOME USEFUL MACROS",
	 "QUELQUES MACROS UTILES");

    a = 0, b = 0, printf("XOR(%d, %d) = %d\n", a, b, XOR(a, b));
    a = 0, b = 1, printf("XOR(%d, %d) = %d\n", a, b, XOR(a, b));
    a = 1, b = 0, printf("XOR(%d, %d) = %d\n", a, b, XOR(a, b));
    a = 1, b = 1, printf("XOR(%d, %d) = %d\n", a, b, XOR(a, b));

    a = -1, PRINT(BOOL, a, d, d, 9);
    a =  0, PRINT(BOOL, a, d, d, 9);
    a = +1, PRINT(BOOL, a, d, d, 9);
    a = +2, PRINT(BOOL, a, d, d, 9);

    a = 0, printf("ISEVEN(%d) = %d, ISODD(%d) = %d\n", a, ISEVEN(a), a, ISODD(a));
    a = 1, printf("ISEVEN(%d) = %d, ISODD(%d) = %d\n", a, ISEVEN(a), a, ISODD(a));
    a = 2, printf("ISEVEN(%d) = %d, ISODD(%d) = %d\n", a, ISEVEN(a), a, ISODD(a));
    a = 3, printf("ISEVEN(%d) = %d, ISODD(%d) = %d\n", a, ISEVEN(a), a, ISODD(a));

    x = -5, PRINT(SIGN, x, +g, +d, 9);
    x =  0, PRINT(SIGN, x, +g, +d, 9);
    x = +3, PRINT(SIGN, x, +g, +d, 9);

    x = -33, PRINT(ABS,  x, g, g, 9);
    x = -5,  PRINT(SQR,  x, g, g, 9);
    x = -3,  PRINT(CUBE, x, g, g, 9);

    x = + 9.9, y = -10, z = +10, printf("CLAMP(%04.1f, %+g, %+g) = %04.1f\n", x, y, z, CLAMP(x, y, z));
    x = +10.0, y = -10, z = +10, printf("CLAMP(%04.1f, %+g, %+g) = %04.1f\n", x, y, z, CLAMP(x, y, z));
    x = +10.1, y = -10, z = +10, printf("CLAMP(%04.1f, %+g, %+g) = %04.1f\n", x, y, z, CLAMP(x, y, z));

    x = 0.00, y = -10, z = +10, printf("LERP(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, LERP(x, y, z));
    x = 0.25, y = -10, z = +10, printf("LERP(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, LERP(x, y, z));
    x = 0.50, y = -10, z = +10, printf("LERP(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, LERP(x, y, z));
    x = 0.75, y = -10, z = +10, printf("LERP(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, LERP(x, y, z));
    x = 1.00, y = -10, z = +10, printf("LERP(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, LERP(x, y, z));

    if (OOR(1.1, -1, +1)) puts("Out Of Range");

    x = 0.9, y = -1., z = +1., printf("OOR(%.2f, %+g, %+g) = %d\n", x, y, z, OOR(x, y, z));
    x = 1.0, y = -1., z = +1., printf("OOR(%.2f, %+g, %+g) = %d\n", x, y, z, OOR(x, y, z));
    x = 1.1, y = -1., z = +1., printf("OOR(%.2f, %+g, %+g) = %d\n", x, y, z, OOR(x, y, z));

    x = 3.14, printf("ROUND(%g) = %d\n", x, ROUND(x));

    x = 11, y = 77, printf("MIN(%g, %g) = %g\n", x, y, MIN(x, y));
    x = 11, y = 77, printf("MAX(%g, %g) = %g\n", x, y, MAX(x, y));

    x = 3.3, y = 9.7;
    printf("XCH(double, %g, %g) ->", x, y);
    XCH(double, x, y);
    printf(" (%g, %g)\n", x, y);

    WAIT();

#if defined __GNUC__ && !defined __STRICT_ANSI__

    COMM("Versions without side effect",
	 "Versions sans effet de bord");

    a = 0, b = 0, printf("Mxor(%d, %d) = %d\n", a, b, Mxor(a, b));
    a = 0, b = 1, printf("Mxor(%d, %d) = %d\n", a, b, Mxor(a, b));
    a = 1, b = 0, printf("Mxor(%d, %d) = %d\n", a, b, Mxor(a, b));
    a = 1, b = 1, printf("Mxor(%d, %d) = %d\n", a, b, Mxor(a, b));

    a = -1, PRINT(Mbool, a, d, d, 9);
    a =  0, PRINT(Mbool, a, d, d, 9);
    a = +1, PRINT(Mbool, a, d, d, 9);
    a = +2, PRINT(Mbool, a, d, d, 9);

    a = 0, printf("Miseven(%d) = %d, Misodd(%d) = %d\n", a, Miseven(a), a, Misodd(a));
    a = 1, printf("Miseven(%d) = %d, Misodd(%d) = %d\n", a, Miseven(a), a, Misodd(a));
    a = 2, printf("Miseven(%d) = %d, Misodd(%d) = %d\n", a, Miseven(a), a, Misodd(a));
    a = 3, printf("Miseven(%d) = %d, Misodd(%d) = %d\n", a, Miseven(a), a, Misodd(a));

    x = -5, PRINT(Msign, x, +g, +d, 10);
    x =  0, PRINT(Msign, x, +g, +d, 10);
    x = +3, PRINT(Msign, x, +g, +d, 10);

    x = -33, PRINT(Mabs,  x, g, g, 10);
    x = -5,  PRINT(Msqr,  x, g, g, 10);
    x = -3,  PRINT(Mcube, x, g, g, 10);

    x = + 9.9, y = -10, z = +10, printf("Mclamp(%04.1f, %+g, %+g) = %04.1f\n", x, y, z, Mclamp(x, y, z));
    x = +10.0, y = -10, z = +10, printf("Mclamp(%04.1f, %+g, %+g) = %04.1f\n", x, y, z, Mclamp(x, y, z));
    x = +10.1, y = -10, z = +10, printf("Mclamp(%04.1f, %+g, %+g) = %04.1f\n", x, y, z, Mclamp(x, y, z));

    x = 0.00, y = -10, z = +10, printf("Mlerp(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, Mlerp(x, y, z));
    x = 0.25, y = -10, z = +10, printf("Mlerp(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, Mlerp(x, y, z));
    x = 0.50, y = -10, z = +10, printf("Mlerp(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, Mlerp(x, y, z));
    x = 0.75, y = -10, z = +10, printf("Mlerp(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, Mlerp(x, y, z));
    x = 1.00, y = -10, z = +10, printf("Mlerp(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, Mlerp(x, y, z));

    if (Moor(1.1, -1, +1)) puts("Out Of Range");

    x = 0.9, y = -1., z = +1., printf("Moor(%.2f, %+g, %+g) = %d\n", x, y, z, Moor(x, y, z));
    x = 1.0, y = -1., z = +1., printf("Moor(%.2f, %+g, %+g) = %d\n", x, y, z, Moor(x, y, z));
    x = 1.1, y = -1., z = +1., printf("Moor(%.2f, %+g, %+g) = %d\n", x, y, z, Moor(x, y, z));

    x = 3.14, printf("Mround(%g) = %d\n", x, Mround(x));

    x = 11, y = 77, printf("Mmin(%g, %g) = %g\n", x, y, Mmin(x, y));
    x = 11, y = 77, printf("Mmax(%g, %g) = %g\n", x, y, Mmax(x, y));

    x = 3.3, y = 9.7;
    printf("Mswap(%g, %g) ->", x, y);
    Mswap(x, y);
    printf(" (%g, %g)\n", x, y);

    WAIT();

#if defined _XCLIB_SOURCE || defined _XC_MACROS_SOURCE

    COMM("Only with _XCLIB_SOURCE or _XC_MACROS_SOURCE",
	 "Seulement avec _XCLIB_SOURCE ou _XC_MACROS_SOURCE");

    a = 0, b = 0, printf("xor(%d, %d) = %d\n", a, b, xor(a, b));
    a = 0, b = 1, printf("xor(%d, %d) = %d\n", a, b, xor(a, b));
    a = 1, b = 0, printf("xor(%d, %d) = %d\n", a, b, xor(a, b));
    a = 1, b = 1, printf("xor(%d, %d) = %d\n", a, b, xor(a, b));

    a = 0, printf("bool(%d) = %d\n", a, bool(a));
    a = 1, printf("bool(%d) = %d\n", a, bool(a));
    a = 2, printf("bool(%d) = %d\n", a, bool(a));
    a = 3, printf("bool(%d) = %d\n", a, bool(a));

    a = 0, printf("iseven(%d) = %d, isodd(%d) = %d\n", a, iseven(a), a, isodd(a));
    a = 1, printf("iseven(%d) = %d, isodd(%d) = %d\n", a, iseven(a), a, isodd(a));
    a = 2, printf("iseven(%d) = %d, isodd(%d) = %d\n", a, iseven(a), a, isodd(a));
    a = 3, printf("iseven(%d) = %d, isodd(%d) = %d\n", a, iseven(a), a, isodd(a));

    x = -5, PRINT(sign, x, +g, +d, 9);
    x =  0, PRINT(sign, x, +g, +d, 9);
    x = +3, PRINT(sign, x, +g, +d, 9);

    x = -33, PRINT(abs,  x, g, g, 9);
    x = -5,  PRINT(sqr,  x, g, g, 9);
    x = -3,  PRINT(cube, x, g, g, 9);

    x = + 9.9, y = -10, z = +10, printf("clamp(%04.1f, %+g, %+g) = %04.1f\n", x, y, z, clamp(x, y, z));
    x = +10.0, y = -10, z = +10, printf("clamp(%04.1f, %+g, %+g) = %04.1f\n", x, y, z, clamp(x, y, z));
    x = +10.1, y = -10, z = +10, printf("clamp(%04.1f, %+g, %+g) = %04.1f\n", x, y, z, clamp(x, y, z));

    x = 0.00, y = -10, z = +10, printf("lerp(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, lerp(x, y, z));
    x = 0.25, y = -10, z = +10, printf("lerp(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, lerp(x, y, z));
    x = 0.50, y = -10, z = +10, printf("lerp(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, lerp(x, y, z));
    x = 0.75, y = -10, z = +10, printf("lerp(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, lerp(x, y, z));
    x = 1.00, y = -10, z = +10, printf("lerp(%.2f, %+g, %+g) = %+5.1f\n", x, y, z, lerp(x, y, z));

    if (oor(1.1, -1, +1)) puts("Out Of Range");

    x = 0.9, y = -1., z = +1., printf("oor(%.2f, %+g, %+g) = %d\n", x, y, z, oor(x, y, z));
    x = 1.0, y = -1., z = +1., printf("oor(%.2f, %+g, %+g) = %d\n", x, y, z, oor(x, y, z));
    x = 1.1, y = -1., z = +1., printf("oor(%.2f, %+g, %+g) = %d\n", x, y, z, oor(x, y, z));

    x = 3.14, printf("round(%g) = %d\n", x, round(x));

    x = 11, y = 77, printf("min(%g, %g) = %g\n", x, y, min(x, y));
    x = 11, y = 77, printf("max(%g, %g) = %g\n", x, y, max(x, y));

    x = 3.3, y = 9.7;
    printf("swap(%g, %g) ->", x, y);
    swap(x, y);
    printf(" (%g, %g)\n", x, y);

    WAIT();

#endif

#endif

    COMM("SOME USEFUL MACROS",
	 "QUELQUES MACROS UTILES");

    printf("DEG2RAD = %g\n", DEG2RAD);
    printf("RAD2DEG = %g\n", RAD2DEG);

    x = PI,    PRINT(rad2deg, x, 06.2f, 06.2f, 16);
    x = 180.0, PRINT(deg2rad, x, 06.2f, 06.2f, 16);

    WAIT();

    return 0;
}


/* End of file */
