/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*  XCLIB 0.3	LLIST:		DOUBLY LINKED LIST			     */


#ifndef __XC_LList_H
#define __XC_LList_H


/* Standard C Library: */
#include <stddef.h>
#include <assert.h>

/* eXtended C Library: */
#include <xc/version.h>		/* VERSION OF EXTENDED C LIBRARY	     */


/*****************************************************************************
 * CUSTOMIZATION PART
 *****************************************************************************/


#ifndef LL_NodeRec
#define LL_NodeRec  LL_NodeRec
typedef struct node LL_NodeRec;
#endif

#ifndef LL_ListRec
#define LL_ListRec  LL_ListRec
typedef struct list LL_ListRec;
#endif

#ifndef LL_Node
#define LL_Node     LL_Node
typedef LL_NodeRec *LL_Node;
#endif

#ifndef LL_List
#define LL_List     LL_List
typedef LL_ListRec *LL_List;
#endif


#ifndef LL_next
#define LL_next	next
#endif

#ifndef LL_prev
#define LL_prev	prev
#endif

#ifndef LL_head
#define LL_head	head
#endif

#ifndef LL_tail
#define LL_tail	tail
#endif


/*****************************************************************************
 * INTERNALS
 *****************************************************************************/


#if defined __GNUC__ && !defined __STRICT_ANSI__
#define LL_TYPEREC(NODEREC)	typeof (NODEREC)
#define LL_TYPE(NODE)		typeof (NODE)
#else
#define LL_TYPEREC(NODEREC)	LL_NodeRec
#define LL_TYPE(NODE)		LL_Node
#endif


#define	__LL_NEXT(NODE)	((NODE)->LL_next)
#define	__LL_PREV(NODE)	((NODE)->LL_prev)
#define	__LL_HEAD(LIST)	((LIST)->LL_head)
#define	__LL_TAIL(LIST)	((LIST)->LL_tail)


#define __LL_FIRST(H, T)	(H)
#define __LL_LAST(H, T)		(T)


#define __LL_SANITY_CHECK(H, T) assert					      \
									      \
 ( ((H) != NULL && (T) != NULL) || ((H) == NULL && (T) == NULL) )


#define _LL_INIT(H, T)							      \
_MBEGIN									      \
    (H) = NULL;								      \
    (T) = NULL;								      \
_MEND

#define _LL_FOREACH_COND(H, T, NODE, COND)				      \
									      \
 for (									      \
									      \
 __LL_SANITY_CHECK(H, T),						      \
									      \
 (NODE) = __LL_FIRST(H, T); (NODE) != NULL && (COND); (NODE) = __LL_NEXT(NODE))

#define _LL_FOREACH(H, T, NODE)	_LL_FOREACH_COND(H, T, NODE, 1)

#define _LL_FOREACH_NODE(H, T, INST)					      \
_MBEGIN									      \
    __XC_REGISTER LL_TYPE(H) __node;					      \
									      \
    _LL_FOREACH(H, T, __node) { INST }					      \
_MEND

#define _LL_NEXT(N)	__LL_NEXT(N)
#define _LL_PREV(N)	__LL_PREV(N)

#define _LL_FIRST(H, T)	(__LL_SANITY_CHECK(H, T), __LL_FIRST(H, T))
#define _LL_LAST(H, T)	(__LL_SANITY_CHECK(H, T), __LL_LAST(H, T))

#define _LL_APPEND(H, T, N)						      \
_MBEGIN									      \
    __LL_SANITY_CHECK(H, T);						      \
									      \
    assert((N) != NULL);						      \
									      \
    __LL_NEXT(N) = NULL;						      \
    __LL_PREV(N) = (T);							      \
									      \
    if ((T) != NULL) __LL_NEXT(T) = (N); else (H) = (N);		      \
									      \
    (T) = (N);								      \
_MEND

#define _LL_INSERT(H, T, N)						      \
_MBEGIN									      \
    __LL_SANITY_CHECK(H, T);						      \
									      \
    assert((N) != NULL);						      \
									      \
    __LL_PREV(N) = NULL;						      \
    __LL_NEXT(N) = (H);							      \
									      \
    if ((H) != NULL) __LL_PREV(H) = (N); else (T) = (N);		      \
									      \
    (H) = (N);								      \
_MEND

#define _LL_PREPEND(H, T, NODE) _LL_INSERT(H, T, NODE)

#define _LL_INSERT_BEFORE(H, T, NODE, MARK)				      \
_MBEGIN									      \
    __LL_SANITY_CHECK(H, T);						      \
									      \
    assert((NODE) != NULL);						      \
    assert((MARK) != NULL);						      \
									      \
    if ((MARK) == (H))							      \
    {									      \
	_LL_INSERT(H, T, NODE);						      \
    }									      \
    else								      \
    {									      \
	__LL_NEXT(NODE) = (MARK);					      \
	__LL_PREV(NODE) = __LL_PREV(MARK);				      \
									      \
	__LL_PREV(__LL_NEXT(NODE)) = (NODE);				      \
	__LL_NEXT(__LL_PREV(NODE)) = (NODE);				      \
    }									      \
_MEND

#define _LL_INSERT_AFTER(H, T, NODE, MARK)				      \
_MBEGIN									      \
    __LL_SANITY_CHECK(H, T);						      \
									      \
    assert((NODE) != NULL);						      \
    assert((MARK) != NULL);						      \
									      \
    if ((MARK) == (T))							      \
    {									      \
	_LL_APPEND(H, T, NODE);						      \
    }									      \
    else								      \
    {									      \
	__LL_PREV(NODE) = (MARK);					      \
	__LL_NEXT(NODE) = __LL_NEXT(MARK);				      \
									      \
	__LL_NEXT(__LL_PREV(NODE)) = (NODE);				      \
	__LL_PREV(__LL_NEXT(NODE)) = (NODE);				      \
    }									      \
_MEND

#define __LL_REMOVE(H, T, NODE)						      \
_MBEGIN									      \
    __LL_SANITY_CHECK(H, T);						      \
									      \
    assert((NODE) != NULL);						      \
									      \
    if ((NODE) == (H))							      \
    {									      \
	if (((H) = __LL_NEXT(H)) != NULL) __LL_PREV(H) = NULL;		      \
    }									      \
    else								      \
    {									      \
	__LL_NEXT(__LL_PREV(NODE)) = __LL_NEXT(NODE);			      \
    }									      \
									      \
    if ((NODE) == (T))							      \
    {									      \
	if (((T) = __LL_PREV(T)) != NULL) __LL_NEXT(T) = NULL;		      \
    }									      \
    else								      \
    {									      \
	__LL_PREV(__LL_NEXT(NODE)) = __LL_PREV(NODE);			      \
    }									      \
									      \
    __LL_NEXT(NODE) = NULL;						      \
    __LL_PREV(NODE) = NULL;						      \
_MEND

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _LL_REMOVE(H, T, NODE)	(__LL_REMOVE(H, T, NODE), (NODE))
#else
#define _LL_REMOVE(H, T, NODE)	__LL_REMOVE(H, T, NODE)
#endif

#define _LL_EXTRACT(H, T, NODE)	_LL_REMOVE(H, T, NODE)

#define _LL_FOREWARD_ACTION(H, T, INST)					      \
_MBEGIN									      \
    __XC_REGISTER LL_TYPE(H) __node, ___next_node;			      \
									      \
    __LL_SANITY_CHECK(H, T);						      \
									      \
    for (__node = __LL_FIRST(H, T); __node != NULL; __node = ___next_node)    \
    {									      \
	___next_node = __LL_NEXT(__node);				      \
									      \
	INST								      \
    }									      \
_MEND

#define _LL_BACKWARD_ACTION(H, T, INST)					      \
_MBEGIN									      \
    __XC_REGISTER LL_TYPE(H) __node, ___prev_node;			      \
									      \
    __LL_SANITY_CHECK(H, T);						      \
									      \
    for (__node = __LL_LAST(H, T); __node != NULL; __node = ___prev_node)     \
    {									      \
	___prev_node = __LL_PREV(__node);				      \
									      \
	INST								      \
    }									      \
_MEND

#define _LL_ACTION(H, T, INST)		_LL_FOREWARD_ACTION(H, T, INST)
#define _LL_FWD_ACTION(H, T, INST)	_LL_FOREWARD_ACTION(H, T, INST)
#define _LL_BWD_ACTION(H, T, INST)	_LL_BACKWARD_ACTION(H, T, INST)

#define _LL_CLEAN(H, T, INST)						      \
_MBEGIN									      \
    _LL_ACTION(H, T, INST);						      \
									      \
    _LL_INIT(H, T);							      \
_MEND

#define _LL_PURGE(H, T, INST)	_LL_PURGE(H, T, INST)

#define __LL_LENGTH(H, T, LEN)						      \
_MBEGIN									      \
    __XC_REGISTER size_t __i = 0;					      \
									      \
    _LL_FOREACH_NODE(H, T, __i++;);					      \
									      \
    (LEN) = __i;							      \
_MEND

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _LL_LENGTH(H, T)						      \
_MBEGIN									      \
    __XC_REGISTER size_t __len;						      \
									      \
    __LL_LENGTH(H, T, __len);						      \
									      \
    __len;								      \
_MEND
#else
#define _LL_LENGTH(H, T, LEN)						      \
_MBEGIN									      \
    __XC_REGISTER size_t __len;						      \
									      \
    __LL_LENGTH(H, T, __len);						      \
									      \
    (LEN) = __len;							      \
_MEND
#endif

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _LL_PLENGTH(H, T, LEN)	((LEN) = _LL_LENGTH(H, T))
#else
#define _LL_PLENGTH(H, T, LEN)	_LL_LENGTH(H, T, LEN)
#endif

#define __LL_SELECT(H, T, INDEX, NODE)					      \
_MBEGIN									      \
    __XC_REGISTER size_t const __i = (INDEX);				      \
    __XC_REGISTER size_t       __j = 0;					      \
									      \
    _LL_FOREACH(H, T, NODE) if (__j++ == __i) break;			      \
_MEND

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _LL_SELECT(H, T, INDEX)						      \
_MBEGIN									      \
    __XC_REGISTER LL_TYPE(H) __node;					      \
									      \
    __LL_SELECT(H, T, INDEX, __node);					      \
									      \
    __node;								      \
_MEND
#else
#define _LL_SELECT(H, T, INDEX, NODE)					      \
_MBEGIN									      \
    __XC_REGISTER LL_TYPE(H) __node;					      \
									      \
    __LL_SELECT(H, T, INDEX, __node);					      \
									      \
    (NODE) = __node;							      \
_MEND
#endif

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _LL_PSELECT(H, T, INDEX, NODE)	((NODE) = _LL_SELECT(H, T, INDEX))
#else
#define _LL_PSELECT(H, T, INDEX, NODE)	_LL_SELECT(H, T, INDEX, NODE)
#endif


/*****************************************************************************
 * HIGH LEVEL MACROS
 *****************************************************************************/


#define _LL_HEAD(LIST)	__LL_HEAD((assert((LIST) != NULL), LIST))
#define _LL_TAIL(LIST)	__LL_TAIL((assert((LIST) != NULL), LIST))


#define LL_INIT(LIST)				_LL_INIT(_LL_HEAD(LIST), _LL_TAIL(LIST))
#define	LL_NEXT(MARK)				_LL_NEXT(MARK)
#define	LL_PREV(MARK)				_LL_PREV(MARK)
#define LL_FIRST(LIST)				_LL_FIRST(_LL_HEAD(LIST), _LL_TAIL(LIST))
#define LL_LAST(LIST)				_LL_LAST(_LL_HEAD(LIST), _LL_TAIL(LIST))
#define LL_APPEND(LIST, NODE)			_LL_APPEND(_LL_HEAD(LIST), _LL_TAIL(LIST), NODE)
#define LL_INSERT(LIST, NODE)			_LL_INSERT(_LL_HEAD(LIST), _LL_TAIL(LIST), NODE)
#define LL_PREPEND(LIST, NODE)			_LL_PREPEND(_LL_HEAD(LIST), _LL_TAIL(LIST), NODE)
#define LL_INSERT_BEFORE(LIST, NODE, MARK)	_LL_INSERT_BEFORE(_LL_HEAD(LIST), _LL_TAIL(LIST), NODE, MARK)
#define LL_INSERT_AFTER(LIST, NODE, MARK)	_LL_INSERT_AFTER(_LL_HEAD(LIST), _LL_TAIL(LIST), NODE, MARK)
#define LL_REMOVE(LIST, NODE)			_LL_REMOVE(_LL_HEAD(LIST), _LL_TAIL(LIST), NODE)
#define LL_EXTRACT(LIST, NODE)			_LL_EXTRACT(_LL_HEAD(LIST), _LL_TAIL(LIST), NODE)
#define LL_FOREACH_NODE(LIST, INST)		_LL_FOREACH_NODE(_LL_HEAD(LIST), _LL_TAIL(LIST), INST)
#define LL_FOREACH(LIST, NODE)			_LL_FOREACH(_LL_HEAD(LIST), _LL_TAIL(LIST), NODE)
#define LL_FOREACH_COND(LIST, NODE, COND)	_LL_FOREACH_COND(_LL_HEAD(LIST), _LL_TAIL(LIST), NODE, COND)
#define LL_ACTION(LIST, INST)			_LL_ACTION(_LL_HEAD(LIST), _LL_TAIL(LIST), INST)
#define LL_FWD_ACTION(LIST, INST)		_LL_FWD_ACTION(_LL_HEAD(LIST), _LL_TAIL(LIST), INST)
#define LL_BWD_ACTION(LIST, INST)		_LL_BWD_ACTION(_LL_HEAD(LIST), _LL_TAIL(LIST), INST)
#define LL_FOREWARD_ACTION(LIST, INST)		_LL_FOREWARD_ACTION(_LL_HEAD(LIST), _LL_TAIL(LIST), INST)
#define LL_BACKWARD_ACTION(LIST, INST)		_LL_BACKWARD_ACTION(_LL_HEAD(LIST), _LL_TAIL(LIST), INST)
#define LL_CLEAN(LIST, INST)			_LL_CLEAN(_LL_HEAD(LIST), _LL_TAIL(LIST), INST)
#define LL_PURGE(LIST, INST)			_LL_PURGE(_LL_HEAD(LIST), _LL_TAIL(LIST), INST)

/*
 * Non portable versions
 */

#if defined __GNUC__ && !defined __STRICT_ANSI__

/* ====> Function behaviour */

#define LL_LENGTH(LIST)				_LL_LENGTH(_LL_HEAD(LIST), _LL_TAIL(LIST))
#define LL_SELECT(LIST, I)			_LL_SELECT(_LL_HEAD(LIST), _LL_TAIL(LIST), I)

#else

/* ====> strict ANSI conformance */

#define LL_LENGTH(LIST, LEN)			_LL_LENGTH( _LL_HEAD(LIST), _LL_TAIL(LIST), LEN)
#define LL_SELECT(LIST, I, NODE)		_LL_SELECT(_LL_HEAD(LIST), _LL_TAIL(LIST), I, NODE)

#endif

/*
 * Portable versions
 */

#define LL_PLENGTH(LIST, LEN)			_LL_PLENGTH(_LL_HEAD(LIST), _LL_TAIL(LIST), LEN)
#define LL_PSELECT(LIST, I, NODE)		_LL_PSELECT(_LL_HEAD(LIST), _LL_TAIL(LIST), I, NODE)


#endif /* XC LList HEADER */


/*------------------------------/ END OF FILE /------------------------------*/
