/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*  XCLIB 0.3	IO:		SYNCHRONOUS I/O MULTIPLEXING		     */


#ifndef __XC_IO_H
#define __XC_IO_H


/* POSIX: */
#include <sys/time.h>

/* eXtended C Library: */
#include <xc/cdecl.h>		/* C DECLARATIONS			     */


enum IOEvent { IO_READ, IO_WRITE, IO_EXCEPT };

enum IOMask
{
    ioREAD   = (1 << IO_READ),
    ioWRITE  = (1 << IO_WRITE),
    ioEXCEPT = (1 << IO_EXCEPT)
};

typedef enum IOEvent      IOEvent;
typedef enum IOMask       IOMask;
typedef struct timeval    IOTime;
typedef unsigned long int IOCard;
typedef void const        IOData;

typedef const struct __IOCheckerNode *IOCheckerId;
typedef const struct __IOHandlerNode *IOHandlerId;
typedef const struct __IOTimerNode   *IOTimerId;
typedef const struct __IOIdlerNode   *IOIdlerId;
typedef const struct __IOSignalNode  *IOSignalId;

typedef void (*IOCheckerProc)(IOCheckerId __id, IOData *__data);
typedef void (*IOHandlerProc)(IOHandlerId __id, int __fd, IOEvent __op, IOData *__data);
typedef void (*IOTimerProc)(IOTimerId __id, IOTime *__spec, IOData *__data);
typedef void (*IOIdlerProc)(IOIdlerId __id, IOCard __count, IOData *__data);
typedef void (*IOSignalProc)(IOSignalId __id, IOData *__data);


_CDECL_BEGIN

IOCheckerId
io_add_checker(IOCheckerProc __proc, IOData *__data);

void
io_del_checker(IOCheckerId __checker);

IOHandlerId
io_add_handler(int __fd, IOMask __mask, IOHandlerProc __proc, IOData *__data);

void
io_del_handler(IOHandlerId __handler);

IOTimerId
io_add_timer(const IOTime *__spec, IOTimerProc __proc, IOData *__data);

IOTimerId
io_add_timeout(unsigned long int __msec, IOTimerProc __proc, IOData *__data);

void
io_del_timer(IOTimerId __timer);

IOIdlerId
io_add_idler(int __freq, IOIdlerProc __proc, IOData *__data);

void
io_del_idler(IOIdlerId __idler);

IOSignalId
io_add_signal(IOSignalProc __proc, IOData *__data);

void
io_del_signal(IOSignalId __signal);

void
io_main_loop(void);

void
io_exit_loop(void);

#if 0

IOCheckerId
io_ctx_ctx_add_checker(IOContext context, IOCheckerProc proc, IOData *data);

void
io_ctx_del_checker(IOContext context, IOCheckerId checker);

IOHandlerId
io_ctx_add_handler(IOContext context, int fd, IOMask mask, IOHandlerProc proc, IOData *data);

void
io_ctx_del_handler(IOContext context, IOHandlerId handler);

IOTimerId
io_ctx_add_timer(IOContext context, const IOTime *spec, IOTimerProc proc, IOData *data);

void
io_ctx_del_timer(IOContext context, IOTimerId timer);

IOIdlerId
io_ctx_add_idler(IOContext context, int freq, IOIdlerProc proc, IOData *data);

void
io_ctx_del_idler(IOContext context, IOIdlerId idler);

IOSignalId
io_ctx_add_signal(IOContext context, IOSignalProc proc, IOData *data);

void
io_ctx_del_signal(IOContext context, IOSignalId signal);

#endif

_CDECL_END


#endif /* XC IO HEADER */


/*------------------------------/ END OF FILE /------------------------------*/
