/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*  XCLIB 0.3	MISSING:	PROBABLY MISSING FUNCTIONS		     */


/*****************************************************************************\

 [English]

 [French]

 L'inclusion de ce fichier pour obtenir les dclarations de certaines fonctions
 ne dispense pas (ou ne devrait pas) dispenser de mettre en oeuvre correctement
 les standards (ou les normes) correspondants.
 Il est sans inutile de rappeler qu'il est toujours souhaitable de matriser le
 contenu d'un standard, et d'viter d'utiliser des fonctionnalits trop
 exotiques. Donc prudence: si vous tes amens  utilisez ce fichier, soit vous
 vous trouvez sur un systme de qualit mdiocre, soit votre source contient
 des parties qui peuvent ventuellement vous causer des difficults (mme si ce
 module de XCLIB est l pour y pallier).

 Pour prvenir tout conflit, s'il exite un homonyme sous la forme d'une macro-
 dfinition, la dclaration de la fonction sera empche. Charge au programmeur
 de faire le mnage avant (avec des `#undef ...' par exemple) si ncessaire.

 NB: A l'usage, il est possible que ce fichier devienne moins draconien...

\*****************************************************************************/


#ifndef __XC_MISSING_H
#define __XC_MISSING_H


/* Standard C Library: */
#include <stddef.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* eXtended C Library: */
#include <xc/cdecl.h>		/* C DECLARATIONS			     */


#if !defined _XC_MISSING_STRING	&&\
    !defined _XC_MISSING_STDIO	&&\
    !defined _XC_MISSING_MATH	&&\
    !defined _XC_MISSING_TGMATH
#define __XC_USE_MISSING_STRING
#define __XC_USE_MISSING_STDIO
#define __XC_USE_MISSING_MATH
#define __XC_USE_MISSING_TGMATH
#else
#ifdef       _XC_MISSING_STRING
#define __XC_USE_MISSING_STRING
#endif
#ifdef       _XC_MISSING_STDIO
#define __XC_USE_MISSING_STDIO
#endif
#ifdef       _XC_MISSING_MATH
#define __XC_USE_MISSING_MATH
#endif
#ifdef       _XC_MISSING_TGMATH
#define __XC_USE_MISSING_TGMATH
#endif
#endif


_CDECL_BEGIN


/*****************************************************************************
 * <string.h> - string manipulation
 *****************************************************************************/

#ifdef __XC_USE_MISSING_STRING
#endif /* I USE MISSING STRING */















#if defined __XC_USE_SVID || defined __XC_USE_BSD || defined __XC_USE_XOPEN_EXTENDED

#ifndef __GLIBC__

#ifndef strdup
char *strdup(const char *s);
#endif

#endif /* not GLIBC */

#endif /* USE C9X if not GLIBC */



/*****************************************************************************
 * <math.h> - common mathematical operations on real floating-point values
 *****************************************************************************/


#ifdef __XC_USE_MISSING_MATH

#if defined __XC_USE_ISOC9X && !defined __GLIBC__

#ifndef fmax
double fmax(double __x, double __y) __attribute__ ((__const__));
#endif
#ifndef fmaxf
float fmaxf(float __x, float __y) __attribute__ ((__const__));
#endif
#ifndef fmaxl
long double fmaxl(long double __x, long double __y) __attribute__ ((__const__));
#endif

#ifndef fmin
double fmin(double __x, double __y) __attribute__ ((__const__));
#endif
#ifndef fminf
float fminf(float __x, float __y) __attribute__ ((__const__));
#endif
#ifndef fminl
long double fminl(long double __x, long double __y) __attribute__ ((__const__));
#endif

#endif /* USE C9X if not GLIBC */

#endif /* USE MISSING MATH */


/*****************************************************************************
 * <tgmath.h> - generic functions
 *****************************************************************************/


#ifdef __XC_USE_MISSING_TGMATH

#if defined __XC_USE_ISOC9X && !defined __GLIBC__

#ifndef fmax
#define fmax(x, y)\
  (sizeof(x) == sizeof(float) && sizeof(y) == sizeof(float)) ?\
    fmaxf(x, y)\
  : (sizeof(x) == sizeof(double) && sizeof(y) == sizeof(double)) ?\
    fmax(x, y)\
  :\
    fmaxl(x, y)
#endif

#ifndef fmin
#define fmin(x, y)\
  (sizeof(x) == sizeof(float) && sizeof(y) == sizeof(float)) ?\
    fminf(x, y)\
  : (sizeof(x) == sizeof(double) && sizeof(y) == sizeof(double)) ?\
    fmin(x, y)\
  :\
    fminl(x, y)
#endif

#endif /* USE C9X if not GLIBC */

#endif /* USE MISSING TGMATH */


#if (defined __XC_USE_ISOC9X || defined __XC_USE_BSD) && !defined __GLIBC__

#ifndef isinf
int isinf(double value);
#define isinf(x)	((sizeof(x) == sizeof(x)) ? isinff(x) : isinf(x))
#endif

#ifndef isnan
int isnan(double value);
#define isnan(x)	((sizeof(x) == sizeof(x)) ? isnanf(x) : isnan(x))
#endif

#ifndef finite
int finite(double value);
#define finite(x)	((sizeof(x) == sizeof(x)) ? finitef(x) : finite(x))
#endif

#endif /* C9X or BSD, but not GLIBC */


#if defined __GLIBC__
#ifdef __STRICT_ANSI__
float fabsf(float x);
float atan2f(float y, float x);
#endif
#else
#define fabsf	__xclib(fabsf)
#define atan2f	__xclib(atan2f)
float __xclib(fabsf)(float x);
float __xclib(atan2f)(float y, float x);
#endif


#if defined _GNU_SOURCE && !defined __GLIBC__

int asprintf(char **pstr, const char *format, ...)
 __attribute__ ((__format__ (__printf__, 2, 3)));
int vasprintf(char **pstr, const char *format, va_list vargs)
 __attribute__ ((__format__ (__printf__, 2, 0)));

#endif


#ifdef __GLIBC__
#ifdef __STRICT_ANSI__
int snprintf(char *buf, size_t size, const char *format, ...)
 __attribute__ ((__format__ (__printf__, 3, 4)));
int vsnprintf(char *buf, size_t size, const char *format, va_list vargs)
 __attribute__ ((__format__ (__printf__, 3, 0)));
#endif
#else
#define snprintf	__xclib(snprintf)
#define vsnprintf	__xclib(vsnprintf)
#endif


int __xclib(snprintf)(char *buf, size_t size, const char *format, ...)
 __attribute__ ((__format__ (__printf__, 3, 4)));
int __xclib(vsnprintf)(char *buf, size_t size, const char *format, va_list vargs)
 __attribute__ ((__format__ (__printf__, 3, 0)));


_CDECL_END


#endif /* XC MISSING HEADER */


/*------------------------------/ END OF FILE /------------------------------*/
