/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*  XCLIB 0.3	TRACE:		TRACE FACILITY				     */


#ifndef __XC_TRACE_H
#define __XC_TRACE_H


/* Standard C Library: */
#include <stdio.h>

/* eXtended C Library: */
#include <xc/cdecl.h>		/* C DECLARATIONS			     */


#if	defined __TRACE__
#if	defined __DEBUG__
#ifdef __MACRO_VARARGS__
#define trace(format, args...)	__xclib(dbg_trace)(__XC_ARGS, format, ##args)
#else
#define trace(text)		__xclib(dbg_trace)(__XC_ARGS, text)
#endif
#define trace_in		__xclib(dbg_trace_in)(__XC_ARGS, __FUNC__)
#define trace_out		__xclib(dbg_trace_out)(__XC_ARGS, __FUNC__)
#elif	defined __FACI__
#ifdef __MACRO_VARARGS__
#define trace(format, args...)	__xclib(f_trace)(__XC_FACI, format, ##args)
#else
#define trace(text)		__xclib(f_trace)(__XC_FACI, text)
#endif
#define trace_in		__xclib(f_trace_in)(__XC_FACI, __FUNC__)
#define trace_out		__xclib(f_trace_out)(__XC_FACI, __FUNC__)
#else
#ifdef __MACRO_VARARGS__
#define trace(format, args...)	__xclib(trace)(format, ##args)
#else
#define trace(text)		__xclib(trace)(text)
#endif
#define trace_in		__xclib(trace_in)(__FUNC__)
#define trace_out		__xclib(trace_out)(__FUNC__)
#endif
#else
#ifdef __MACRO_VARARGS__
#define trace(format, args...)	__noop
#else
#define trace(text)		__noop
#endif
#define trace_in		__noop
#define trace_out		__noop
#endif

#ifdef __TRACE__
#define __trace_in__	trace_in;{
#define __trace_out__	}trace_out;
#else
#define __trace_in__
#define __trace_out__
#endif


#define set_trace_output(stream)	__xclib(set_trace_output)(stream)


_CDECL_BEGIN

void __xclib(trace)(const char *format, ...)
 __attribute__ ((__format__ (__printf__, 1, 2)));
void __xclib(trace_in)(const char *function, ...)
 __attribute__ ((__format__ (__printf__, 1, 2)));
void __xclib(trace_out)(const char *function, ...)
 __attribute__ ((__format__ (__printf__, 1, 2)));

void __xclib(f_trace)(__xc_faci, const char *format, ...)
 __attribute__ ((__format__ (__printf__, 2, 3)));
void __xclib(f_trace_in)(__xc_faci, const char *function, ...)
 __attribute__ ((__format__ (__printf__, 2, 3)));
void __xclib(f_trace_out)(__xc_faci, const char *function, ...)
 __attribute__ ((__format__ (__printf__, 2, 3)));

void __xclib(dbg_trace)(__xc_args, const char *format, ...)
 __attribute__ ((__format__ (__printf__, 4, 5)));
void __xclib(dbg_trace_in)(__xc_args, const char *function, ...)
 __attribute__ ((__format__ (__printf__, 4, 5)));
void __xclib(dbg_trace_out)(__xc_args, const char *function, ...)
 __attribute__ ((__format__ (__printf__, 4, 5)));

void __xclib(set_trace_output)(FILE *stream);

_CDECL_END


#endif /* XC TRACE HEADER */


/*------------------------------/ END OF FILE /------------------------------*/
