/*  XCLIB - a set of C functions for general purposes		      -*- c -*-
    Copyright (C) 1996-2001  Martin Vicente

    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*\
  |  Martin Vicente, 1996-2001
  |
  | Project : XCLIB (eXtended C Library)
  | Purpose : PRIMARY FUNCTIONS
  | Version : 0.3
  | Language: ANSI C (with GNU things)
  | File    : base/get_ansi_color_seq.c
  | Author  : Martin VICENTE <martin@vicente.org>
\*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*/


#define _XCLIB_INTERNALS

#define _GNU_SOURCE

#include "config.h"

#if !defined __GNUC__ || defined __STRICT_ANSI__
#  if !HAVE_ALLOCA_H
#    ifdef _AIX
#      pragma alloca
#    endif
#  endif
#endif

#include <stddef.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#if !defined __GNUC__ || defined __STRICT_ANSI__
#  if HAVE_ALLOCA_H
#    include <alloca.h>
#  else
#    ifndef _AIX
#      ifndef alloca
         extern char *alloca();
#      endif
#    endif
#  endif
#endif

#include <xc/colors.h>
#include <xc/strlib.h>

#include "xc/base.h"

#include "global.h"

#define EQ(s1, s2)	(strcmp(s1, s2) == 0)

const char *get_ansi_color_seq(const char *const color, int const foreground)
{
    char *tmp;

    assert(color != NULL);

    if (color[0] == '\0') return "";

    if ((tmp = strdupa(color)) == NULL) panic();

    strupp(tmp);

    if (EQ(tmp, "0"))     return NULL;
    if (EQ(tmp, "NO"))    return NULL;
    if (EQ(tmp, "FALSE")) return NULL;

    if (foreground)
    {
	if (EQ(tmp, "BLACK"))   return SGR_BLACK;
	if (EQ(tmp, "RED"))     return SGR_RED;
	if (EQ(tmp, "GREEN"))   return SGR_GREEN;
	if (EQ(tmp, "YELLOW"))  return SGR_YELLOW;
	if (EQ(tmp, "BLUE"))    return SGR_BLUE;
	if (EQ(tmp, "MAGENTA")) return SGR_MAGENTA;
	if (EQ(tmp, "CYAN"))    return SGR_CYAN;
	if (EQ(tmp, "WHITE"))   return SGR_WHITE;
    }
    else
    {
	if (EQ(tmp, "BLACK"))   return SGR_BG_BLACK;
	if (EQ(tmp, "RED"))     return SGR_BG_RED;
	if (EQ(tmp, "GREEN"))   return SGR_BG_GREEN;
	if (EQ(tmp, "YELLOW"))  return SGR_BG_YELLOW;
	if (EQ(tmp, "BLUE"))    return SGR_BG_BLUE;
	if (EQ(tmp, "MAGENTA")) return SGR_BG_MAGENTA;
	if (EQ(tmp, "CYAN"))    return SGR_BG_CYAN;
	if (EQ(tmp, "WHITE"))   return SGR_BG_WHITE;
    }

    return "";
}


/* End Of File */
