/*  XCLIB - a set of C functions for general purposes		      -*- c -*-
    Copyright (C) 1996-2001  Martin Vicente

    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*\
  |  Martin Vicente, 1996-2001
  |
  | Project : XCLIB (eXtended C Library)
  | Purpose : POSIX TIME SERVICE
  | Version : 0.3
  | Language: ANSI C
  | File    : time/fix.c
  | Author  : Martin VICENTE <martin@vicente.org>
\*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*/


/*****************************************************************************\

 EDOM    Math argument out of domain of function
 ERANGE  Math result not representable

\*****************************************************************************/


#include "time/common.h"

int fix_time(struct timeval *t)
{
    int sec;

    assert(t != NULL);

    if (USEC(t) > 0)
    {
	while (USEC(t) >= +MILLION)
	{
	    if (SEC(t) == TIME_MAX)
	    {
		errno = ERANGE;

		return ERROR;
	    }

	    SEC(t)++;

	    USEC(t) -= MILLION;
	}

	if (USEC(t) > 0 && SEC(t) < 0)
	{
	    SEC(t)++;

	    USEC(t) -= MILLION;
	}
    }
    else if (USEC(t) < 0)
    {
	while (USEC(t) <= -MILLION)
	{
	    if (SEC(t) == TIME_MIN)
	    {
		errno = ERANGE;

		return ERROR;
	    }

	    SEC(t)--;

	    USEC(t) += MILLION;
	}

	if (USEC(t) < 0 && SEC(t) > 0)
	{
	    SEC(t)--;

	    USEC(t) += MILLION;
	}
    }

    return OK;
}


/* End Of File */
