/*  XCLIB - a set of C functions for general purposes		      -*- c -*-
    Copyright (C) 1996-2001  Martin Vicente

    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*\
  |  Martin Vicente, 1996-2001
  |
  | Project : XCLIB (eXtended C Library)
  | Purpose : POSIX TIME SERVICE
  | Version : 0.3
  | Language: ANSI C
  | File    : time/sub.c
  | Author  : Martin VICENTE <martin@vicente.org>
\*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*/


/*****************************************************************************\

 EDOM    Math argument out of domain of function
 ERANGE  Math result not representable

 [English]

 [French]

 L'algorithme utilis est lgrement restrictif.

\*****************************************************************************/


#include "time/common.h"

int sub_time(struct timeval *t, const struct timeval *t2)
{
    struct timeval __t2, *const _t2 = &__t2;

    assert(t != NULL && t2 != NULL);

    __t2 = *t2;

#ifdef __DEBUG__
    if (fix_time(t) == ERROR || fix_time(_t2) == ERROR)
    {
	errno = EDOM;

	return ERROR;
    }
#endif

    /**/ if (USEC(t) > 0 && USEC(_t2) < 0)
    {
	if (USEC(t) > TIME_MAX + USEC(_t2))
	{
	    errno = ERANGE;

	    return ERROR;
	}
    }
    else if (USEC(t) < 0 && USEC(_t2) > 0)
    {
	if (USEC(t) < TIME_MIN + USEC(_t2))
	{
	    errno = ERANGE;

	    return ERROR;
	}
    }

    /**/ if (SEC(t) > 0 && SEC(_t2) < 0)
    {
	if (SEC(t) > TIME_MAX + SEC(_t2))
	{
	    errno = ERANGE;

	    return ERROR;
	}
    }
    else if (SEC(t) < 0 && SEC(_t2) > 0)
    {
	if (SEC(t) < TIME_MIN + SEC(_t2))
	{
	    errno = ERANGE;

	    return ERROR;
	}
    }

    USEC(t) -= USEC(_t2);
    SEC(t)  -= SEC(_t2);

    if (fix_time(t) == ERROR)
    {
	errno = ERANGE;

	return ERROR;
    }

    return OK;
}


/* End Of File */
