############################################################## -*- makefile -*-
#
#	Copyright (C) 1996-2001  Martin Vicente
#	This file is part of the eXtended C Library (XCLIB).
#
#	This library is free software; you can redistribute it and/or
#	modify it under the terms of the GNU Library General Public
#	License as published by the Free Software Foundation; either
#	version 2 of the License, or (at your option) any later version.
#
#	This library is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#	Library General Public License for more details.
#
#	You should have received a copy of the GNU Library General Public
#	License along with this library (see the file COPYING.LIB).
#	If not, write to the Free Software Foundation, Inc.,
#	59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
###############################################################################
#
#	INSTALLATION
#
# Warning: This is an include file for GNU makefiles only.
#
#	Project : "Les outils de Martin"
#	Purpose : Installation
#	File    : utils/install.mk
#	Version : 1.1
#	Language: GNU Make
#	Author  : Martin Vicente
#	Contact : martin@vicente.org (or martin.vicente@free.fr)
#	Revision: Mon, 14 May 2001
#
###############################################################################

ifneq (,)
This makefile requires GNU Make.
endif

ifndef   __UTILS_INSTALL_MK
override __UTILS_INSTALL_MK := OK

#==============================================================================

include $(UTILS)/mklog.mk

#====( DEFINITIONS )

override INSTALL_HEADERS =\
	if [ -n "$(VERBOSE)" ]; then verbose=true; else verbose=false; fi;    \
	directory="$(subst /,\/,$(shell dirname $<)/)";			      \
	for file in $^; do						      \
	    dest=`echo $$file | sed s/^$$directory//`;			      \
	    if $$verbose; then						      \
		$(LOG) "\t"$$dest$(_C) $(END);				      \
	    fi;								      \
	    dest=$(includedir)/$$dest;					      \
	    $(MKDIR) `dirname $$dest` && $(INSTALL_DATA) $$file $$dest;	      \
	done

# EN TEST
override INSTALL_HEADERS =\
	if [ -n "$(VERBOSE)" ]; then verbose=true; else verbose=false; fi;    \
	directory="$(subst /,\/,$(shell dirname $<)/)";			      \
	for file in $^; do						      \
	    dest=`echo $$file | sed s/^$$directory//`;			      \
	    if [ -n "`dirname $< | grep -e '/$(MAINDIR)$$'`" ]; then	      \
		if $$verbose; then					      \
		    $(LOG) "\t"$(MAINDIR)/$$dest$(_C) $(END);		      \
		fi;							      \
		dest=$(includedir)/$(MAINDIR)/$$dest;			      \
	    else							      \
		if $$verbose; then					      \
		    $(LOG) "\t"$$dest$(_C) $(END);			      \
		fi;							      \
		dest=$(includedir)/$$dest;				      \
	    fi;								      \
	    $(MKDIR) `dirname $$dest` && $(INSTALL_DATA) $$file $$dest;	      \
	done

override INSTALL_ARCHIVE =						      \
									      \
	if [ "$(DEVEL)" = 1 ];						      \
	then $(MKDIR) $(libdir) && $(INSTALL_DATA) -m 640 $< $(libdir);	      \
	else $(MKDIR) $(libdir) && $(INSTALL_DATA) -m 644 $< $(libdir);	      \
	fi

override INSTALL_SHLIB =						      \
									      \
	if [ "$(DEVEL)" = 1 ];						      \
	then $(MKDIR) $(libdir) && $(INSTALL_DATA) -m 640 $< $(libdir);	      \
	else $(MKDIR) $(libdir) && $(INSTALL_DATA) -m 644 $< $(libdir);	      \
	fi;								      \
	strip --strip-unneeded $(libdir)/`basename $<`

override INSTALL_PROGRAMS =						      \
									      \
	$(MKDIR) $(bindir);						      \
	if [ -n "$(VERBOSE)" ]; then verbose=true; else verbose=false; fi;    \
	directory="$(subst /,\/,$(shell dirname $<)/)";			      \
	for file in $^; do						      \
	    dest=`echo $$file | sed s/^$$directory//`;			      \
	    if $$verbose; then						      \
		$(LOG) "\t"$$dest$(_C) $(END);				      \
	    fi;								      \
	    $(INSTALL_PROGRAM) $$file $(bindir)/`echo $$dest | sed '$(transform)'`;\
	done

override INSTALL_MANUALS_1 =						      \
									      \
	ddir=$(mandir)/man1;						      \
									      \
	$(MKDIR) $$ddir;						      \
	if [ -n "$(VERBOSE)" ]; then verbose=true; else verbose=false; fi;    \
	directory="$(subst /,\/,$(shell dirname $<)/)";			      \
	for file in $^; do						      \
	    dest=`echo $$file | sed s/^$$directory//`;			      \
	    if $$verbose; then						      \
		$(LOG) "\t"$$dest$(_C) $(END);				      \
	    fi;								      \
	    $(INSTALL_DATA) $$file $$ddir/`echo $$dest | sed '$(transform)'`; \
	done

override INSTALL_MANUALS_1_FR =						      \
									      \
	ddir=$(mandir)/fr/man1;						      \
									      \
	$(MKDIR) $$ddir;						      \
	if [ -n "$(VERBOSE)" ]; then verbose=true; else verbose=false; fi;    \
	directory="$(subst /,\/,$(shell dirname $<)/)";			      \
	for file in $^; do						      \
	    dest=`echo $$file | sed s/^$$directory//`;			      \
	    if $$verbose; then						      \
		$(LOG) "\t"$$dest$(_C) $(END);				      \
	    fi;								      \
	    $(INSTALL_DATA) $$file $$ddir/`echo $$dest | sed '$(transform)'`; \
	done

override INSTALL_MANUALS_3 =						      \
									      \
	ddir=$(mandir)/man3;						      \
									      \
	$(MKDIR) $$ddir;						      \
	if [ -n "$(VERBOSE)" ]; then verbose=true; else verbose=false; fi;    \
	directory="$(subst /,\/,$(shell dirname $<)/)";			      \
	for file in $^; do						      \
	    dest=`echo $$file | sed s/^$$directory//`;			      \
	    if $$verbose; then						      \
		$(LOG) "\t"$$dest$(_C) $(END);				      \
	    fi;								      \
	    $(INSTALL_DATA) $$file $$ddir/$$dest;			      \
	done

override INSTALL_MANUALS_3_FR =						      \
									      \
	ddir=$(mandir)/fr/man3;						      \
									      \
	$(MKDIR) $$ddir;						      \
	if [ -n "$(VERBOSE)" ]; then verbose=true; else verbose=false; fi;    \
	directory="$(subst /,\/,$(shell dirname $<)/)";			      \
	for file in $^; do						      \
	    dest=`echo $$file | sed s/^$$directory//`;			      \
	    if $$verbose; then						      \
		$(LOG) "\t"$$dest$(_C) $(END);				      \
	    fi;								      \
	    $(INSTALL_DATA) $$file $$ddir/$$dest;			      \
	done

override INSTALL_EXAMPLES =						      \
									      \
	ddir=$(examplesdir);						      \
									      \
	$(MKDIR) $$ddir;						      \
	if [ -n "$(VERBOSE)" ]; then verbose=true; else verbose=false; fi;    \
	directory="$(subst /,\/,$(shell dirname $<)/)";			      \
	for file in $^; do						      \
	    dest=`echo $$file | sed s/^$$directory//`;			      \
	    if $$verbose; then						      \
		$(LOG) "\t"$$dest$(_C) $(END);				      \
	    fi;								      \
	    $(INSTALL_DATA) $$file $$ddir/$$dest;			      \
	done

#==============================================================================

endif  # __UTILS_INSTALL_MK

# [END OF FILE]
