############################################################## -*- makefile -*-
#
#	Copyright (C) 1996-2001  Martin Vicente
#	This file is part of the eXtended C Library (XCLIB).
#
#	This library is free software; you can redistribute it and/or
#	modify it under the terms of the GNU Library General Public
#	License as published by the Free Software Foundation; either
#	version 2 of the License, or (at your option) any later version.
#
#	This library is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#	Library General Public License for more details.
#
#	You should have received a copy of the GNU Library General Public
#	License along with this library (see the file COPYING.LIB).
#	If not, write to the Free Software Foundation, Inc.,
#	59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
###############################################################################
#
#	MAKE LOG (for verbose mode)
#
# Warning: This is an include file for GNU makefiles only.
#
#	Project : "Les outils de Martin"
#	Purpose : To log messages with GNU makefile
#	File    : utils/mklog.mk
#	Version : 1.1
#	Language: GNU Make
#	Author  : Martin Vicente
#	Contact : martin@vicente.org (or martin.vicente@free.fr)
#	Revision: Mon, 14 May 2001
#
# Description:
#
#	This file defines log and error messages.
#
# Environment:
#
#	VERBOSE = No/Yes
#
###############################################################################

ifneq (,)
This makefile requires GNU Make.
endif

ifndef   __UTILS_MKLOG_MK
override __UTILS_MKLOG_MK := OK

#==============================================================================

override UTILS ?= $(HOME)/devel/utils

include $(UTILS)/utils.mk
include $(UTILS)/color.mk

#====( CUSTOMIZATION )

ifdef XMESSAGE
override XLOG ?= $(XMESSAGE) -center -timeout 1 -bg Orange -fg Blue  -buttons ""
override XERR ?= $(XMESSAGE) -center -timeout 0 -bg Red    -fg Black -buttons Cancel:0
endif

#====( DEFINITIONS )

ifdef VERBOSE
  ifeq ($(VERBOSE), 0)
    override VERBOSE :=
  else
    override VERBOSE := $(shell echo $(VERBOSE) | $(LOWER))
  ifeq ($(VERBOSE), false)
    override VERBOSE :=
  else
  ifeq ($(VERBOSE), no)
    override VERBOSE :=
  else
  ifeq ($(VERBOSE), off)
    override VERBOSE :=
  else
    override VERBOSE := Yes
  endif
  endif
  endif
  endif
endif

override PROMPT := 'make:'
override ERROR  := 'make: *** [error]'

ifndef color

  ifdef PACKAGE
    override PROMPT += '[$(PACKAGE)]'
  endif

  override LOG :=     $(ECHO_E) "\n"$(PROMPT)
  override ERR := >&2 $(ECHO_E) "\n"$(ERROR)
  override END :=

  override Log =     $(ECHO_E) '\n'$(PROMPT) $1
  override Err = >&2 $(ECHO_E) '\n'$(ERROR)  $1

else

  ifdef PACKAGE
    override PROMPT += '['$(bold)'$(PACKAGE)'$(bold_off)']'
  endif

  ifndef bg_color

    override LOG :=     $(ECHO_E) $(_N) $(reset)$(color)$(PROMPT)
    override ERR := >&2 $(ECHO_E) $(_N) $(reset)$(red)$(ERROR)
    override END := $(_C) && $(ECHO_E) $(reset)

    override Log =     $(ECHO_E) $(reset)$(call Color, $(PROMPT) $1)
    override Err = >&2 $(ECHO_E) $(reset)$(call Red, $(ERROR) $(call Bold, $1))

  else

    override BG10 := '          '
    override BG80 := $(BG10)$(BG10)$(BG10)$(BG10)$(BG10)$(BG10)$(BG10)$(BG10)

    override Background = $(bg_color)$(BG80)'\r'$(strip $1)$(bg_default)

    override LOG :=     $(ECHO_E) $(_N) $(reset)$(bg_color)$(BG80)'\r'$(color)$(PROMPT)
    override ERR := >&2 $(ECHO_E) $(_N) $(reset)$(bg_black)$(BG80)'\r'$(red)$(ERROR)
    override END := $(_C) && $(ECHO_E) $(reset)

    override Log = >&1 $(ECHO_E) $(reset)$(call Background, $(call Color, $(PROMPT) $1))
    override Err = >&2 $(ECHO_E) $(reset)$(call BgBlack, $(call Red, $(ERROR) $(call Bold, $1)))

  endif

endif

override Background ?= $(strip $1)

ifdef DISPLAY
ifdef XLOG
override Log += ; ( if test -z $(findstring \033, $1); then $(XLOG) $1; fi & )
endif
ifdef XERR
override Err += ; ( if test -z $(findstring \033, $1); then $(XERR) $1; fi )
endif
endif

override Err += ; exit 1

#====( MESSAGES )

ifdef VERBOSE

override ~makedep     := "MAKEDEP"
override ~compile     := "COMPILE"
override ~compile_pic := "COMPILE (PIC)"
override ~sharlib_pic := "SHARLIB (PIC)"
override ~link        := "LINK ->"
override ~archive     := "ARCHIVE"

override ~create = "Creating $@"
override ~ready  = "$@ is ready"

override ~install-headers    := "Installing headers in"
override ~install-archive    := "Installing archive file in"
override ~install-library    := "Installing shared library in"
override ~install-man1-en    := "Installing english manuals \(section 1\) in"
override ~install-man3-en    := "Installing english manuals \(section 3\) in"
override ~install-man1-fr    := "Installing french manuals \(section 1\) in"
override ~install-man3-fr    := "Installing french manuals \(section 3\) in"
override ~install-programs   := "Installing demo programs in"
override ~install-examples   := "Installing examples in"
override ~uninstall-headers  := "Uninstalling headers from"
override ~uninstall-archive  := "Uninstalling archive from"
override ~uninstall-library  := "Uninstalling shared library from"
override ~uninstall-man1-en  := "Uninstalling english manuals \(section 1\) from"
override ~uninstall-man3-en  := "Uninstalling english manuals \(section 3\) from"
override ~uninstall-man1-fr  := "Uninstalling french manuals \(section 1\) from"
override ~uninstall-man3-fr  := "Uninstalling french manuals \(section 3\) from"
override ~uninstall-programs := "Uninstalling demo programs from"
override ~uninstall-examples := "Uninstalling examples from"

override ~clean     := "Deleting programs"
override ~realclean := "Deleting binaries"
override ~distclean := "Cleaning distribution"
override ~autoconf  := "AUTOMATIC REMAKING (Autoconf) ->"
endif

#==============================================================================

endif  # __UTILS_MKLOG_MK

# [END OF FILE]
